@@ -13,6 +13,22 @@ from rich.console import Console
 from rich.table import Table
 
 from ._version import __version__
+from .close_manager import AsyncCloseManager
+from .error_codes import (
+    DependamergeError,
+    ExitCode,
+    convert_git_error,
+    convert_github_api_error,
+    convert_network_error,
+    exit_for_configuration_error,
+    exit_for_github_api_error,
+    exit_for_pr_state_error,
+    exit_with_error,
+    is_github_api_permission_error,
+    is_network_error,
+)
+from .git_ops import GitError
+from .github_async import GraphQLError, RateLimitError, SecondaryRateLimitError
 from .github_client import GitHubClient
 from .merge_manager import AsyncMergeManager
 from .models import PullRequestInfo
@@ -36,8 +52,6 @@ class CustomTyper(typer.Typer):
 
     def __call__(self, *args, **kwargs):
         # Check if help is being requested
-        import sys
-
         if "--help" in sys.argv or "-h" in sys.argv:
             console.print(f"🏷️  dependamerge version {__version__}")
         return super().__call__(*args, **kwargs)
@@ -207,6 +221,11 @@ def merge(
         "--dismiss-copilot",
         help="Automatically dismiss unresolved GitHub Copilot review comments",
     ),
+    force: str = typer.Option(
+        "none",
+        "--force",
+        help="Override level: 'none', 'code-owners', 'protection-rules', 'all' (use with caution)",
+    ),
 ):
     """
     Bulk approve/merge pull requests across a GitHub organization.
@@ -227,7 +246,26 @@ def merge(
     Merges similar PRs from the same automation tool (dependabot, pre-commit.ci).
 
     For user generated bulk PRs, use the --override flag with SHA hash.
+
+    Force levels:
+    - none: Respect all protections (default)
+    - code-owners: Bypass code owner review requirements
+    - protection-rules: Bypass branch protection checks (requires permissions)
+    - all: Attempt merge despite most warnings (not recommended)
     """
+    # Validate force level
+    valid_force_levels = ["none", "code-owners", "protection-rules", "all"]
+    if force not in valid_force_levels:
+        console.print(
+            f"Error: Invalid --force level '{force}'. Must be one of: {', '.join(valid_force_levels)}"
+        )
+        raise typer.Exit(1)
+
+    # Warn user about force levels
+    if force == "all":
+        console.print("⚠️  Warning: Using --force=all will bypass most safety checks.")
+        console.print("   This may attempt merges that will fail at GitHub API level.")
+
     # Initialize progress tracker
     progress_tracker = None
 
@@ -263,8 +301,9 @@ def merge(
             if source_pr.state != "open":
                 if progress_tracker:
                     progress_tracker.stop()
-                console.print(f"🛑 Failed: {pr_url} [already closed]")
-                raise typer.Exit(1)
+                exit_for_pr_state_error(
+                    pr_number, "closed", details="Pull request has been closed"
+                )
         except (
             urllib3.exceptions.NameResolutionError,
             urllib3.exceptions.MaxRetryError,
@@ -272,12 +311,23 @@ def merge(
             requests.exceptions.Timeout,
             requests.exceptions.RequestException,
         ) as e:
-            console.print(
-                "Network Error: Failed to connect to GitHub API while fetching source PR."
-            )
-            console.print(f"Details: {e}")
-            console.print("Please check your internet connection and try again.")
-            raise typer.Exit(1) from e
+            if is_network_error(e):
+                exit_with_error(
+                    ExitCode.NETWORK_ERROR,
+                    details="Failed to fetch PR details from GitHub API",
+                    exception=e,
+                )
+            elif is_github_api_permission_error(e):
+                exit_for_github_api_error(
+                    details="Failed to fetch PR details", exception=e
+                )
+            else:
+                exit_with_error(
+                    ExitCode.GENERAL_ERROR,
+                    message="❌ Failed to fetch PR details",
+                    details=str(e),
+                    exception=e,
+                )
 
         # Display source PR info
         _display_pr_info(
@@ -286,7 +336,7 @@ def merge(
 
         # Debug matching info for source PR
         if debug_matching:
-            console.print("\n🔍 [bold]Debug Matching Information[/bold]")
+            console.print("\n🔍 Debug Matching Information")
             console.print(
                 f"   Source PR automation status: {github_client.is_automation_author(source_pr.author)}"
             )
@@ -329,11 +379,12 @@ def merge(
 
             # Validate provided override SHA
             if not _validate_override_sha(override, source_pr, first_commit_line):
-                console.print(
-                    "Error: Invalid override SHA. Expected SHA for this PR and author is:"
+                # Use the already generated expected_sha for error message
+                exit_with_error(
+                    ExitCode.VALIDATION_ERROR,
+                    message="❌ Invalid override SHA provided",
+                    details=f"Expected SHA for this PR and author: --override {expected_sha}",
                 )
-                console.print(f"--override {expected_sha}")
-                raise typer.Exit(1)
 
             console.print(
                 "Override SHA validated. Proceeding with non-automation PR merge."
@@ -354,12 +405,23 @@ def merge(
             requests.exceptions.Timeout,
             requests.exceptions.RequestException,
         ) as e:
-            console.print(
-                "Network Error: Failed to connect to GitHub API while fetching organization repositories."
-            )
-            console.print(f"Details: {e}")
-            console.print("Please check your internet connection and try again.")
-            raise typer.Exit(1) from e
+            if is_network_error(e):
+                exit_with_error(
+                    ExitCode.NETWORK_ERROR,
+                    details="Failed to fetch organization repositories from GitHub API",
+                    exception=e,
+                )
+            elif is_github_api_permission_error(e):
+                exit_for_github_api_error(
+                    details="Failed to fetch organization repositories", exception=e
+                )
+            else:
+                exit_with_error(
+                    ExitCode.GENERAL_ERROR,
+                    message="❌ Failed to fetch organization repositories",
+                    details=str(e),
+                    exception=e,
+                )
         console.print(f"Found {len(repositories)} repositories")
         #     progress.update(task, description=f"Found {len(repositories)} repositories")
 
@@ -450,6 +512,7 @@ def merge(
                 progress_tracker=progress_tracker,
                 dry_run=not no_confirm,
                 dismiss_copilot=dismiss_copilot,
+                force_level=force,
             ) as merge_manager:
                 if not no_confirm:
                     pass  # No merge message in dry-run mode
@@ -523,6 +586,7 @@ def merge(
                                     progress_tracker=progress_tracker,
                                     dry_run=False,  # Real merge this time
                                     dismiss_copilot=dismiss_copilot,
+                                    force_level=force,
                                 ) as real_merge_manager:
                                     return await real_merge_manager.merge_prs_parallel(
                                         mergeable_prs
@@ -585,18 +649,49 @@ def merge(
         else:
             console.print("❌ No PRs were processed")
 
+    except DependamergeError as exc:
+        # Our structured errors handle display and exit themselves
+        if progress_tracker:
+            progress_tracker.stop()
+        exc.display_and_exit()
+    except (KeyboardInterrupt, SystemExit):
+        # Don't catch system interrupts or exits
+        if progress_tracker:
+            progress_tracker.stop()
+        raise
     except typer.Exit:
         # Handle typer exits (like closed PR errors) gracefully - already printed message
         if progress_tracker:
             progress_tracker.stop()
         # Re-raise without additional error messages
         raise
+    except (GitError, RateLimitError, SecondaryRateLimitError, GraphQLError) as exc:
+        # Convert known errors to centralized error handling
+        if progress_tracker:
+            progress_tracker.stop()
+        if isinstance(exc, GitError):
+            converted_error = convert_git_error(exc)
+        else:  # GitHub API errors
+            converted_error = convert_github_api_error(exc)
+        converted_error.display_and_exit()
     except Exception as e:
         # Ensure progress tracker is stopped even if an unexpected error occurs
         if progress_tracker:
             progress_tracker.stop()
-        console.print(f"❌ Unexpected error: {e}")
-        raise typer.Exit(1) from e
+
+        # Try to categorize the error
+        if is_github_api_permission_error(e):
+            exit_for_github_api_error(exception=e)
+        elif is_network_error(e):
+            converted_error = convert_network_error(e)
+            converted_error.display_and_exit()
+        else:
+            exit_with_error(
+                ExitCode.GENERAL_ERROR,
+                message="❌ Error during merge operation",
+                details=str(e),
+                exception=e,
+            )
 
 
 def _display_pr_info(
@@ -629,6 +724,374 @@ def _display_pr_info(
         progress_tracker.resume()
 
 
+@app.command()
+def close(
+    pr_url: str = typer.Argument(..., help="GitHub pull request URL"),
+    no_confirm: bool = typer.Option(
+        False,
+        "--no-confirm",
+        help="Skip confirmation prompt and close immediately without dry-run",
+    ),
+    similarity_threshold: float = typer.Option(
+        0.8, "--threshold", help="Similarity threshold for matching PRs (0.0-1.0)"
+    ),
+    token: str | None = typer.Option(
+        None, "--token", help="GitHub token (or set GITHUB_TOKEN env var)"
+    ),
+    override: str | None = typer.Option(
+        None, "--override", help="SHA hash to override non-automation PR restriction"
+    ),
+    show_progress: bool = typer.Option(
+        True, "--progress/--no-progress", help="Show real-time progress updates"
+    ),
+    debug_matching: bool = typer.Option(
+        False,
+        "--debug-matching",
+        help="Show detailed scoring information for PR matching",
+    ),
+):
+    """
+    Bulk close pull requests across a GitHub organization.
+
+    By default, runs in interactive mode showing what changes will apply,
+    then prompts to proceed with closing. Use --no-confirm to close immediately.
+
+    This command will:
+
+    1. Analyze the provided PR
+
+    2. Find similar PRs in the organization
+
+    3. Close matching PRs
+
+    Closes similar PRs from the same automation tool (dependabot, pre-commit.ci).
+
+    For user generated bulk PRs, use the --override flag with SHA hash.
+    """
+    # Initialize progress tracker
+    progress_tracker = None
+
+    try:
+        # Parse PR URL first to get organization info
+        github_client = GitHubClient(token)
+        owner, repo_name, pr_number = github_client.parse_pr_url(pr_url)
+
+        if show_progress:
+            progress_tracker = MergeProgressTracker(owner, is_close_operation=True)
+            progress_tracker.start()
+            # Check if Rich display is available
+            if not progress_tracker.rich_available:
+                console.print(f"🔍 Examining source pull request in {owner}...")
+                console.print("Progress updates will be shown as simple text...")
+        else:
+            console.print(f"🔍 Examining source pull request in {owner}...")
+
+        # Get source PR details
+        source_pr = github_client.get_pull_request_info(owner, repo_name, pr_number)
+
+        # Display source PR info
+        _display_pr_info(
+            source_pr, "", github_client, progress_tracker=progress_tracker
+        )
+
+        # Initialize comparator
+        comparator = PRComparator(similarity_threshold)
+
+        # Debug matching info for source PR
+        if debug_matching:
+            console.print("\n🔍 Debug Matching Information")
+            console.print(
+                f"   Source PR automation status: {github_client.is_automation_author(source_pr.author)}"
+            )
+            console.print(
+                f"   Extracted package: '{comparator._extract_package_name(source_pr.title)}'"
+            )
+            console.print(f"   Similarity threshold: {similarity_threshold}")
+            if source_pr.body:
+                console.print(f"   Body preview: {source_pr.body[:100]}...")
+                console.print(
+                    f"   Is dependabot body: {comparator._is_dependabot_body(source_pr.body)}"
+                )
+            else:
+                console.print("   ⚠️  Source PR has no body")
+            console.print()
+
+        # Check if source PR is from automation or has valid override
+        is_automation = github_client.is_automation_author(source_pr.author)
+        override_valid = False
+
+        if not is_automation:
+            # Get first commit message for SHA generation
+            commit_messages = github_client.get_pull_request_commits(
+                owner, repo_name, pr_number
+            )
+            first_commit_line = (
+                commit_messages[0].split("\n")[0] if commit_messages else ""
+            )
+
+            # Generate expected SHA
+            expected_sha = _generate_override_sha(source_pr, first_commit_line)
+
+            # Check if override matches
+            if override == expected_sha:
+                override_valid = True
+
+            if not override:
+                console.print("Source PR is not from a recognized automation tool.")
+                console.print(
+                    f"To close this and similar PRs, run again with: --override {expected_sha}"
+                )
+                console.print(
+                    f"This SHA is based on the author '{source_pr.author}' and commit message '{first_commit_line[:50]}...'",
+                    style="dim",
+                )
+                return
+
+            if not override_valid:
+                console.print(
+                    f"Error: Invalid override SHA. Expected: {expected_sha}",
+                    style="bold red",
+                )
+                console.print(
+                    "This prevents accidental bulk operations on non-automation PRs.",
+                    style="dim",
+                )
+                return
+
+            console.print(
+                "Override SHA validated. Proceeding with non-automation PR close."
+            )
+
+        # Find similar PRs in the organization
+        if progress_tracker:
+            console.print()
+        else:
+            console.print(f"\nChecking organization: {owner}")
+
+        # Use GitHubService for async PR finding
+        from .github_service import GitHubService
+
+        if progress_tracker:
+            progress_tracker.update_operation("Listing repositories...")
+
+        async def _find_similar():
+            svc = GitHubService(
+                token=token,
+                progress_tracker=progress_tracker,
+                debug_matching=debug_matching,
+            )
+            try:
+                only_automation = github_client.is_automation_author(source_pr.author)
+                return await svc.find_similar_prs(
+                    owner,
+                    source_pr,
+                    comparator,
+                    only_automation=only_automation,
+                )
+            finally:
+                await svc.close()
+
+        all_similar_prs = asyncio.run(_find_similar())
+
+        # Stop progress tracker before displaying results
+        if progress_tracker:
+            progress_tracker.stop()
+            summary = progress_tracker.get_summary()
+            elapsed_time = summary.get("elapsed_time")
+            total_prs_analyzed = summary.get("total_prs_analyzed")
+            completed_repositories = summary.get("completed_repositories")
+            similar_prs_found = summary.get("similar_prs_found")
+            errors_count = summary.get("errors_count", 0)
+            console.print(f"\n✅ Analysis completed in {elapsed_time}")
+            console.print(
+                f"📊 Analyzed {total_prs_analyzed} PRs across {completed_repositories} repositories"
+            )
+            console.print(f"🔍 Found {similar_prs_found} similar PRs")
+            if errors_count > 0:
+                console.print(f"⚠️  {errors_count} errors encountered during analysis")
+            console.print()
+
+        if not all_similar_prs:
+            console.print("❌ No similar PRs found in the organization")
+
+        console.print(f"Found {len(all_similar_prs)} similar PRs:")
+
+        for target_pr, comparison in all_similar_prs:
+            console.print(f"  • {target_pr.repository_full_name} #{target_pr.number}")
+            console.print(f"    {_format_condensed_similarity(comparison)}")
+
+        if not no_confirm:
+            # IMPORTANT: Each PR must produce exactly ONE line of output in this section
+            console.print("\n🔍 Dependamerge Evaluation\n")
+
+        # Determine which PRs to close
+        all_prs_to_close = [source_pr] + [pr for pr, _ in all_similar_prs]
+
+        # Perform dry-run close operation
+        async def _close_parallel(prs, dry_run_mode):
+            close_manager = AsyncCloseManager(
+                token=token,
+                progress_tracker=progress_tracker,
+                dry_run=dry_run_mode,
+            )
+            async with close_manager:
+                # Convert to list of tuples (PR, None) for consistency
+                pr_tuples = [(pr, None) for pr in prs]
+                return await close_manager.close_prs_parallel(pr_tuples)
+
+        # Perform dry-run to check which PRs can be closed
+        if not no_confirm:
+            if progress_tracker:
+                progress_tracker.start()
+                console.print()
+            else:
+                console.print(
+                    f"\n🚀 Evaluating {len(all_prs_to_close)} pull requests..."
+                )
+
+            close_results = asyncio.run(_close_parallel(all_prs_to_close, True))
+
+            if progress_tracker:
+                progress_tracker.stop()
+                console.print()
+
+            # Count closeable PRs
+            closed_count = sum(1 for r in close_results if r.status.value == "closed")
+            total_to_close = len(all_prs_to_close)
+
+            if not no_confirm:
+                console.print(f"\nCloseable {closed_count}/{total_to_close} PRs")
+
+                # Generate continuation SHA and prompt user
+                if closed_count > 0:
+                    # Get commit message for SHA generation
+                    commit_messages = github_client.get_pull_request_commits(
+                        owner, repo_name, pr_number
+                    )
+                    first_commit_line = (
+                        commit_messages[0].split("\n")[0] if commit_messages else ""
+                    )
+                    continue_sha_hash = _generate_continue_sha(
+                        source_pr, first_commit_line
+                    )
+                    console.print()
+                    console.print(f"To proceed with closing enter: {continue_sha_hash}")
+
+                    # Check if in test mode (don't prompt during tests)
+                    if "pytest" in sys.modules or os.getenv("TESTING"):
+                        console.print(
+                            "⚠️  Test mode detected - skipping interactive prompt"
+                        )
+                        return
+
+                    user_input = typer.prompt(
+                        "\nEnter the string above to continue (or press Enter to cancel)"
+                    ).strip()
+
+                    if user_input == continue_sha_hash:
+                        # Run actual close on closeable PRs only
+                        console.print(
+                            f"\n🔨 Closing {closed_count} closeable pull requests..."
+                        )
+                        closeable_prs = []
+                        for i, result in enumerate(close_results):
+                            if (
+                                result.status.value == "closed"
+                            ):  # These were dry-run "closed"
+                                closeable_prs.append(all_prs_to_close[i])
+
+                        if progress_tracker:
+                            progress_tracker.start()
+
+                        final_results = asyncio.run(
+                            _close_parallel(closeable_prs, False)
+                        )
+
+                        if progress_tracker:
+                            progress_tracker.stop()
+
+                        # Count final results
+                        final_closed = sum(
+                            1 for r in final_results if r.status.value == "closed"
+                        )
+                        final_failed = sum(
+                            1 for r in final_results if r.status.value == "failed"
+                        )
+
+                        console.print(
+                            f"\n🚀 Final Results: {final_closed} closed, {final_failed} failed"
+                        )
+
+                    else:
+                        console.print("\n❌ Operation cancelled by user")
+                        return
+                else:
+                    console.print("\n❌ No PRs are eligible for closing")
+                    return
+        else:
+            # No confirmation - close immediately
+            if progress_tracker:
+                progress_tracker.start()
+            console.print(f"\n🚀 Closing {len(all_prs_to_close)} pull requests...")
+
+            close_results = asyncio.run(_close_parallel(all_prs_to_close, False))
+
+            if progress_tracker:
+                progress_tracker.stop()
+
+            # Count results
+            closed_count = sum(1 for r in close_results if r.status.value == "closed")
+            failed_count = sum(1 for r in close_results if r.status.value == "failed")
+
+            console.print(
+                f"\n🚀 Final Results: {closed_count} closed, {failed_count} failed"
+            )
+
+    except DependamergeError as exc:
+        # Our structured errors handle display and exit themselves
+        if progress_tracker:
+            progress_tracker.stop()
+        exc.display_and_exit()
+    except (KeyboardInterrupt, SystemExit):
+        # Don't catch system interrupts or exits
+        if progress_tracker:
+            progress_tracker.stop()
+        raise
+    except typer.Exit:
+        # Handle typer exits gracefully - already printed message
+        if progress_tracker:
+            progress_tracker.stop()
+        # Re-raise without additional error messages
+        raise
+    except (GitError, RateLimitError, SecondaryRateLimitError, GraphQLError) as exc:
+        # Convert known errors to centralized error handling
+        if progress_tracker:
+            progress_tracker.stop()
+        if isinstance(exc, GitError):
+            converted_error = convert_git_error(exc)
+        else:  # GitHub API errors
+            converted_error = convert_github_api_error(exc)
+        converted_error.display_and_exit()
+    except Exception as e:
+        # Ensure progress tracker is stopped even if an unexpected error occurs
+        if progress_tracker:
+            progress_tracker.stop()
+
+        # Try to categorize the error
+        if is_github_api_permission_error(e):
+            exit_for_github_api_error(exception=e)
+        elif is_network_error(e):
+            converted_error = convert_network_error(e)
+            converted_error.display_and_exit()
+        else:
+            exit_with_error(
+                ExitCode.GENERAL_ERROR,
+                message="❌ Error during close operation",
+                details=str(e),
+                exception=e,
+            )
+
+
 @app.command()
 def blocked(
     organization: str = typer.Argument(
@@ -773,10 +1236,10 @@ def blocked(
 
             token_to_use = token or os.getenv("GITHUB_TOKEN")
             if not token_to_use:
-                console.print(
-                    "A GitHub token is required for --fix. Provide --token or set GITHUB_TOKEN."
+                exit_for_configuration_error(
+                    message="❌ GitHub token required for --fix option",
+                    details="Provide --token or set GITHUB_TOKEN environment variable",
                 )
-                raise typer.Exit(1)
 
             console.print(f"Starting interactive fix for {len(selections)} PR(s)...")
             try:
@@ -800,20 +1263,55 @@ def blocked(
                     f"✅ Fix complete: {success_count}/{len(selections)} succeeded"
                 )
             except Exception as e:
-                console.print(f"Error during fix workflow: {e}")
-                raise typer.Exit(1) from e
+                exit_with_error(
+                    ExitCode.GENERAL_ERROR,
+                    message="❌ Error during fix workflow",
+                    details=str(e),
+                    exception=e,
+                )
 
+    except DependamergeError as exc:
+        # Our structured errors handle display and exit themselves
+        if progress_tracker:
+            progress_tracker.stop()
+        exc.display_and_exit()
+    except (KeyboardInterrupt, SystemExit):
+        # Don't catch system interrupts or exits
+        if progress_tracker:
+            progress_tracker.stop()
+        raise
     except typer.Exit as e:
         # Handle typer exits gracefully
         if progress_tracker:
             progress_tracker.stop()
         raise e
+    except (GitError, RateLimitError, SecondaryRateLimitError, GraphQLError) as exc:
+        # Convert known errors to centralized error handling
+        if progress_tracker:
+            progress_tracker.stop()
+        if isinstance(exc, GitError):
+            converted_error = convert_git_error(exc)
+        else:  # GitHub API errors
+            converted_error = convert_github_api_error(exc)
+        converted_error.display_and_exit()
     except Exception as e:
         # Ensure progress tracker is stopped even if an error occurs
         if progress_tracker:
             progress_tracker.stop()
-        console.print(f"❌ Unexpected error: {e}")
-        raise typer.Exit(1) from e
+
+        # Try to categorize the error
+        if is_github_api_permission_error(e):
+            exit_for_github_api_error(exception=e)
+        elif is_network_error(e):
+            converted_error = convert_network_error(e)
+            converted_error.display_and_exit()
+        else:
+            exit_with_error(
+                ExitCode.GENERAL_ERROR,
+                message="❌ Error during organization scan",
+                details=str(e),
+                exception=e,
+            )
 
 
 def _display_blocked_results(scan_result, output_format: str):
